<?php

/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    www.mergado.cz
 * @copyright 2016 Mergado technologies, s. r. o.
 * @license   LICENSE.txt
 */

namespace Mergado\Tools;

use DateTime;
use DOMDocument;
use Mergado_Marketing_Pack_Admin;
use XMLWriter;

class XMLStockFeed
{
    protected $language;
    protected $currency;
    protected $logger;
    protected $token;
    protected $lock;

    protected $currentBlogId;

    // Dirs
    public $tmpBlogDir;
    public $stockFeedTmpDir;
    public $outputBlogDir;
    public $stockFeedOutputDir;

    /*******************************************************************************************************************
     * XML GENERATORS
     *******************************************************************************************************************/
    /*******************************************************************************************************************
     * HEUREKA STOCK FEED
     *******************************************************************************************************************/

    public function __construct()
    {
        $this->logger = wc_get_logger();
        $this->token = Mergado_Marketing_Pack_Admin::getToken();
        $this->currentBlogId = Settings::getCurrentBlogId();
        $this->lock = get_option('stock_feed_' . $this->token);

        $this->tmpBlogDir = __MERGADO_TMP_DIR__ . $this->currentBlogId;
        $this->stockFeedTmpDir = __MERGADO_TMP_DIR__ . $this->currentBlogId . '/stockFeed/';
        $this->outputBlogDir =  __MERGADO_XML_DIR__ . $this->currentBlogId;
        $this->stockFeedOutputDir =  __MERGADO_XML_DIR__ . $this->currentBlogId . '/';
    }

    /**
     * @param $page
     * @param bool $redirect
     * @return bool
     */
    public function generateStockXML($page, $redirect = false)
    {
        if (get_option(Settings::HEUREKA['STOCK_FEED'])) {
            $now = new DateTime();
            $this->createNecessaryDirs();

            if($this->isFeedLocked($now)) {
                $this->logger->info('STOCK FEED LOCKED - generating process can\'t proceed');
                throw new CronRunningException();
            } else {
                $this->setFeedLocked($now);

                $productsPerStep = XMLClass::getProductsPerStep('feed-form-stock');
                $currentFilesCount = XMLClass::getCurrentTempFilesCount($this->stockFeedTmpDir);
                $start = XMLClass::getStart($currentFilesCount);

                // If no temporary files, reset generating
                if ($start === 1) {
                    $this->resetFeedGenerating();
                }

                $productsList = XMLClass::getProducts($start, $productsPerStep);

                // Step generating
                if ($this->isPartial($productsPerStep, $productsList)) {
                    $file = $this->stockFeedTmpDir . ($currentFilesCount) . '.xml';

                    $this->logger->info('Mergado log: Stock feed generator started - step ' . $currentFilesCount);
                    $this->createXML($file, $start, $productsPerStep, $productsList);
                    $this->logger->info('Mergado log: Stock feed generator ended - step ' . $currentFilesCount);
                    $this->logger->info('Mergado log: Stock feed generator saved XML file - step ' . $currentFilesCount);

                    $this->increaseIterator();
                    $this->unlockFeed();

                    if ($redirect) {
                        wp_redirect('admin.php?page=' . $page . '&flash=stepGenerated');
                        exit;
                    }
                // Normal generating
                } else if ($this->isNormal($productsPerStep, $productsList)) {
                    $file = $this->stockFeedOutputDir . 'stock_' . $this->token . '.xml';

                    $this->logger->info('Mergado log: Stock feed generator started');
                    $this->createXML($file);
                    $this->logger->info('Mergado log: Stock feed generator ended');
                    $this->logger->info('Mergado log: Stock feed generator saved XML file');

                    $this->unlockFeed();

                    if ($redirect) {
                        wp_redirect('admin.php?page=' . $page . '&flash=stockGenerated');
                        exit;
                    }
                // Merge
                } else {
                    $this->mergeTemporaryFiles();
                    $this->unlockFeed();
                    $this->updateFeedCount();

                    if ($redirect) {
                        wp_redirect('admin.php?page=' . $page . '&flash=mergeGenerated');
                        exit;
                    }
                }

                return true;
            }
        } else {
            $this->logger->info('Mergado log: Heureka stock feed not enabled in settings');
            return false;
        }
    }


    private function createXML($file, $start = null, $limit = null, $products = null)
    {
        if ($products === null) {
            $products = XMLClass::getProducts($start, $limit);
        }

        $xml_new = new XMLWriter();
        $xml_new->openUri($file);
        $xml_new->startDocument('1.0', 'UTF-8');
        $xml_new->startElement('item_list');

        foreach ($products as $i) {
            $product = wc_get_product($i['id']);

            if ($product->is_type('simple')) {
                $qty = $product->get_stock_quantity();
                $stockStatus = ($product->get_stock_status() == 'instock' ? true : false);

                if ($qty <= 0 && !$stockStatus) {
                    continue;
                } elseif ($stockStatus) {
                    if ($qty <= 0) {
                        // Item instock without quantity declared -> heureka needs bigger than 0
                        $qty = 1;
                    }
                    $xml_new->startElement('item');
                    $xml_new->writeAttribute('id', $product->get_id());

                    $xml_new->startElement('stock_quantity');
                    $xml_new->text($qty);
                    $xml_new->endElement();

                    $xml_new->endElement();
                }

            } elseif ($product->is_type('variable')) {
                $variations = $product->get_available_variations();

                if ($variations != []) {
                    foreach ($variations as $variation) {
                        $qty = max($variation['max_qty'], $variation['min_qty']);
                        $stockStatus = $variation['is_in_stock'];

                        if ($qty <= 0 && !$stockStatus) {
                            continue;
                        } elseif ($stockStatus) {
                            if ($qty <= 0) {
                                // Item instock without quantity declared -> heureka needs bigger than 0
                                $qty = 1;
                            }

                            $xml_new->startElement('item');
                            $xml_new->writeAttribute('id', $product->get_id() . '-' . $variation['variation_id']);
                            $xml_new->startElement('stock_quantity');
                            $xml_new->text($qty);
                            $xml_new->endElement();

                            $xml_new->endElement();
                        }
                    }
                }
            }
        }

        $xml_new->endElement();
        $xml_new->endDocument();
        $xml_new->flush();
        unset($xml_new);
    }

    /*******************************************************************************************************************
     * FEED OPTIONS
     *******************************************************************************************************************/

    /**
     * Merge files, create XML and delete temporary files
     */
    private function mergeTemporaryFiles()
    {
        $storage = $this->stockFeedOutputDir . 'stock_' . $this->token . '.xml';
        $tmpShopDir = $this->stockFeedTmpDir;

        $this->logger->info('Merging XML files of stock feed.');
        $loop = 0;

        $xmlstr = '<item_list>';

        foreach (glob($tmpShopDir . '*.xml') as $file) {
            $xml = simplexml_load_file($file);
            foreach ($xml as $item) {
                $xmlstr .= $item->asXml();
            }

            $loop++;
        }

        $xmlstr .= '</item_list>';

        $xml_new = new XMLWriter();

        $xml_new->openURI($storage);
        $xml_new->startDocument('1.0', 'UTF-8');
        $xml_new->writeRaw($xmlstr);
        $xml_new->endDocument();

        $this->logger->info('Stock feed merged. XML created.');


        XMLClass::deleteTemporaryFiles($this->stockFeedTmpDir);

        return true;
    }

    private function isPartial($stepProducts, $productsList)
    {
        return XMLClass::isPartial($stepProducts, $productsList);
    }

    private function isNormal($stepProducts, $productsList)
    {
        return XMLClass::isNormal($stepProducts, $productsList);
    }

    /**
     * Reset feed and delete all TMP files
     */
    private function resetFeedGenerating()
    {
        XMLClass::resetFeedGenerating('mergado-stock-feed-iterator', $this->stockFeedTmpDir);
    }

    /*******************************************************************************************************************
     * FEED LOCKING
     *******************************************************************************************************************/

    /**
     * Return if feed is currently locked
     *
     * @param $now
     * @return bool
     */
    private function isFeedLocked($now)
    {
        return XMLClass::isFeedLocked($this->lock, $now);
    }

    /**
     * Lock feed
     * @param $now
     */
    private function setFeedLocked( $now)
    {
        XMLClass::setFeedLocked('stock_feed_' . $this->token, $now);
        $this->logger->info('STOCK FEED - locking');
    }

    /**
     * Unlock feed
     */
    private function unlockFeed() {
        XMLClass::unlockFeed('stock_feed_' . $this->token);
        $this->logger->info('STOCK FEED - unlocking');
    }

    /*******************************************************************************************************************
     * FEED PRODUCT COUNT
     *******************************************************************************************************************/

    /**
     * Save the feed count to database for next runs
     */
    private function updateFeedCount()
    {
        update_option('mergado-stock-feed-count', (int)get_option('mergado-stock-feed-iterator'));
    }

    private function increaseIterator()
    {
        update_option('mergado-stock-feed-iterator', (int)get_option('mergado-stock-feed-iterator') + 1);
    }

    /*******************************************************************************************************************
     * FEED FOLDER MANIPULATION
     *******************************************************************************************************************/

    /**
     * Check and create necessary directories for this cron
     */
    private function createNecessaryDirs()
    {
        Mergado_Marketing_Pack_Admin::checkAndCreateTmpDataDir();
        Mergado_Marketing_Pack_Admin::createDir($this->tmpBlogDir);
        Mergado_Marketing_Pack_Admin::createDir($this->stockFeedTmpDir);
        Mergado_Marketing_Pack_Admin::createDir($this->outputBlogDir);
        Mergado_Marketing_Pack_Admin::createDir($this->stockFeedOutputDir);
    }

    /*******************************************************************************************************************
     * DATA FOR TEMPLATES
     *******************************************************************************************************************/

    public static function getDataForTemplates()
    {
        if((int)get_option('feed-form-stock', 0) === 0) {
            $totalFiles = 0;
        } else {
            $productsCount = (int)get_option('mergado-stock-feed-count');
            $productsCount = $productsCount === 0 ? wp_count_posts('product')->publish : $productsCount;

            $totalFiles = ceil($productsCount / (int)get_option('feed-form-stock', 0));
        }

        $currentStep = XMLClass::getCurrentTempFilesCount(__MERGADO_TMP_DIR__ . Settings::getCurrentBlogId() . '/stockFeed/');
        $stepProducts = (int) get_option('feed-form-stock', 0);

        if ($totalFiles == 0) {
            $percentageStep = 0;
        } else {
            $percentageStep = round(($currentStep / ($totalFiles)) * 100, 2);
        }

        $isPartiallyGenerated =  (isset($totalFiles) && $totalFiles != 0 && $totalFiles != '');

        return [
            'totalFiles' => $totalFiles,
            'currentStep' => $currentStep,
            'stepProducts' => $stepProducts,
            'percentageStep' => $percentageStep,
            'isPartiallyGenerated' => $isPartiallyGenerated
        ];
    }
}
